@@ -22,7 +22,9 @@ import sphinx.environment
 import sphinx.util.logging
 import sphinx.util.typing
 
-from sphinx_immaterial.apidoc import apidoc_formatting
+from sphinx_immaterial.apidoc import (
+    object_description_options as _object_description_options,
+)
 
 logger = sphinx.util.logging.getLogger(__name__)
 
@@ -56,6 +58,8 @@ extensions = [
     "sphinx_immaterial.apidoc.cpp.cppreference",
     "sphinx_immaterial.apidoc.json.domain",
     "sphinx_immaterial.apidoc.python.apigen",
+    "sphinx_immaterial.apidoc.cpp.apigen",
+    "sphinx_immaterial.graphviz",
     "sphinx_jinja",
 ]
 
@@ -102,13 +106,13 @@ html_theme = "sphinx_immaterial"
 html_theme_options = {
     "icon": {
         "repo": "fontawesome/brands/github",
+        "edit": "material/file-edit-outline",
     },
     "site_url": "https://jbms.github.io/sphinx-immaterial/",
     "repo_url": "https://github.com/jbms/sphinx-immaterial/",
     "repo_name": "Sphinx-Immaterial",
     "repo_type": "github",
     "edit_uri": "blob/main/docs",
-    # "google_analytics": ["UA-XXXXX", "auto"],
     "globaltoc_collapse": True,
     "features": [
         "navigation.expand",
@@ -123,6 +127,8 @@ html_theme_options = {
         "search.share",
         "toc.follow",
         "toc.sticky",
+        "content.tabs.link",
+        "announce.dismiss",
     ],
     "palette": [
         {
@@ -162,6 +168,18 @@ html_theme_options = {
     ],
     # END: version_dropdown
     "toc_title_is_page_title": True,
+    # BEGIN: social icons
+    "social": [
+        {
+            "icon": "fontawesome/brands/github",
+            "link": "https://github.com/jbms/sphinx-immaterial",
+        },
+        {
+            "icon": "fontawesome/brands/python",
+            "link": "https://pypi.org/project/sphinx-immaterial/",
+        },
+    ],
+    # END: social icons
 }
 # end html_theme_options
 
@@ -182,6 +200,14 @@ extlinks = {
         "http://docutils.sourceforge.net/docs/ref/rst/directives.html#%s",
         "rST directive %s",
     ),
+    "graphvizattr": (
+        "https://graphviz.org/docs/attrs/%s/",
+        "%s attribute",
+    ),
+    "dutree": (
+        "https://docutils.sourceforge.io/docs/ref/doctree.html#%s",
+        "%s",
+    ),
 }
 
 object_description_options = []
@@ -233,6 +259,18 @@ rst_prolog = """
 .. role:: rst(code)
    :language: rst
    :class: highlight
+
+.. role:: css(code)
+   :language: css
+   :class: highlight
+
+.. role:: dot(code)
+   :language: dot
+   :class: highlight
+
+.. role:: html(code)
+   :language: html
+   :class: highlight
 """
 
 
@@ -289,6 +327,14 @@ json_schemas = [
     "apidoc/json/inheritance_schema.yml",
 ]
 
+json_schema_rst_prolog = """
+.. default-role:: json:schema
+
+.. default-literal-role:: json
+
+.. highlight:: json
+"""
+
 python_apigen_modules = {"tensorstore_demo": "python_apigen_generated/"}
 
 python_apigen_default_groups = [
@@ -298,24 +344,59 @@ python_apigen_default_groups = [
     (r".*:.*\.__(str|repr)__", "String representation"),
 ]
 
+python_apigen_rst_prolog = """
+.. default-role:: py:obj
+
+.. default-literal-role:: python
+
+.. highlight:: python
+"""
+
+cpp_demo_include_dir = os.path.join(os.path.dirname(__file__))
+
+cpp_apigen_configs = [
+    dict(
+        document_prefix="cpp_apigen_generated/",
+        api_parser_config=dict(
+            input_content="""
+#include "cpp_apigen_demo/index_interval.h"
+#include "cpp_apigen_demo/array.h"
+""",
+            compiler_flags=["-std=c++17", "-I", cpp_demo_include_dir, "-x", "c++"],
+            include_directory_map={
+                f"{cpp_demo_include_dir}/": "",
+            },
+            allow_paths=["^cpp_apigen_demo/"],
+            disallow_namespaces=["^std$"],
+            verbose=True,
+        ),
+    ),
+]
+
+cpp_apigen_rst_prolog = """
+.. default-role:: cpp:expr
+
+.. default-literal-role:: cpp
+
+.. highlight:: cpp
+"""
+
+autodoc_class_signature = "separated"
+
 nitpicky = True
 nitpick_ignore = [
     # Python standard library types not present in object inventory.
-    ('py:class', 'Pattern'),
-    ('py:class', 're.Pattern'),
-
+    ("py:class", "Pattern"),
+    ("py:class", "re.Pattern"),
     # Config option type
-    ('py:class', 'ExternalCppReference'),
-
+    ("py:class", "ExternalCppReference"),
     # Example Python types
-    ('py:class', 'example_mod.Foo'),
-    ('py:class', 'alias_ex.MyUnqualifiedType'),
-
+    ("py:class", "example_mod.Foo"),
+    ("py:class", "alias_ex.MyUnqualifiedType"),
     # Example C++ types
-    ('cpp:identifier', 'Sphinx'),
-    ('cpp:identifier', 'RF24_SPI_SPEED'),
-    ('cpp:identifier', 'RF24_SPI_SPEED'),
-
+    ("cpp:identifier", "Sphinx"),
+    ("cpp:identifier", "RF24_SPI_SPEED"),
+    ("cpp:identifier", "RF24_SPI_SPEED"),
     # C++ namespaces referenced in the documentation
     #
     # It is a bug in the C++ domain that a reference to `ns::symbol` will
@@ -323,19 +404,30 @@ nitpick_ignore = [
     # because the C++ domain does not actually define "namespace" objects, the
     # `ns` reference will always fail to be resolved, leading to a spurious
     # warning.
-    ('cpp:identifier', '::nlohmann'),
-    ('cpp:identifier', 'std'),
-    ('cpp:identifier', 'synopses_ex'),
-    ('cpp:identifier', 'my_ns1'),
-    ('cpp:identifier', 'my_ns2'),
-    ('cpp:identifier', 'my_ns2::my_nested_ns'),
-    ('cpp:identifier', 'my_ns3'),
-
+    ("cpp:identifier", "::nlohmann"),
+    ("cpp:identifier", "std"),
+    ("cpp:identifier", "synopses_ex"),
+    ("cpp:identifier", "my_ns1"),
+    ("cpp:identifier", "my_ns2"),
+    ("cpp:identifier", "my_ns2::my_nested_ns"),
+    ("cpp:identifier", "my_ns3"),
+    ("cpp:identifier", "cpp_apigen_demo"),
+    ("cpp:identifier", "::nlohmann"),
+    ("cpp:identifier", "std"),
+    ("cpp:identifier", "synopses_ex"),
+    ("cpp:identifier", "my_ns1"),
+    ("cpp:identifier", "my_ns2"),
+    ("cpp:identifier", "my_ns2::my_nested_ns"),
+    ("cpp:identifier", "my_ns3"),
     # Example JavaScript types
-    ('js:func', 'string'),
-    ('js:func', 'SomeError'),
+    ("js:func", "string"),
+    ("js:func", "SomeError"),
 ]
 
+
+graphviz_ignore_incorrect_font_metrics = True
+
+
 def _validate_parallel_build(app):
     # Verifies that all of the extensions defined by this theme support parallel
     # building.
@@ -346,7 +438,9 @@ def _validate_parallel_build(app):
 def _parse_object_description_signature(
     env: sphinx.environment.BuildEnvironment, signature: str, node: docutils.nodes.Node
 ) -> str:
-    registry = apidoc_formatting.get_object_description_option_registry(env.app)
+    registry = _object_description_options.get_object_description_option_registry(
+        env.app
+    )
     registry_option = registry.get(signature)
     node += sphinx.addnodes.desc_name(signature, signature)
     if registry_option is None:
