@@ -16,15 +16,12 @@ display the signatures nicely.
 
 import copy
 import dataclasses
-import glob
 import hashlib
 import importlib
 import inspect
 import json
 import os
-import pathlib
 import re
-import secrets
 from typing import (
     List,
     Tuple,
@@ -56,8 +53,10 @@ import sphinx.util.inspect
 import sphinx.util.logging
 import sphinx.util.typing
 
-from .. import apidoc_formatting
+from .. import object_description_options
 from ... import sphinx_utils
+from .. import apigen_utils
+from ... import default_literal_role
 
 logger = sphinx.util.logging.getLogger(__name__)
 
@@ -220,7 +219,7 @@ def _summarize_signature(
 
     obj_desc = node.parent
 
-    options = apidoc_formatting.get_object_description_options(
+    options = object_description_options.get_object_description_options(
         env, obj_desc["domain"], obj_desc["objtype"]
     )
     column_limit = options["wrap_signatures_column_limit"]
@@ -587,20 +586,41 @@ def _generate_entity_desc_node(
     else:
         options["canonical"] = entity.canonical_object_name
     try:
-        nodes = [
-            x
-            for x in sphinx_utils.parse_rst(
-                state=state,
-                text=sphinx_utils.format_directive(
-                    entity.directive,
-                    signatures=[name + sig for sig in entity.signatures],
-                    content="\n".join(content),
-                    options=options,
-                ),
-                source_path=entity.object_name,
+        with apigen_utils.save_rst_defaults(env):
+            rst_input = docutils.statemachine.StringList()
+            sphinx_utils.append_multiline_string_to_stringlist(
+                rst_input,
+                ".. highlight-push::\n\n"
+                + env.config.python_apigen_rst_prolog
+                + "\n\n",
+                "<python_apigen_rst_prolog>",
+                -2,
             )
-            if isinstance(x, sphinx.addnodes.desc)
-        ]
+            sphinx_utils.append_directive_to_stringlist(
+                rst_input,
+                entity.directive,
+                signatures=[name + sig for sig in entity.signatures],
+                content="\n".join(content),
+                source_path=entity.object_name,
+                source_line=0,
+                options=options,
+            )
+            sphinx_utils.append_multiline_string_to_stringlist(
+                rst_input,
+                "\n\n"
+                + env.config.python_apigen_rst_epilog
+                + "\n\n.. highlight-pop::\n\n",
+                "<python_apigen_rst_epilog>",
+                -2,
+            )
+            nodes = [
+                x
+                for x in sphinx_utils.parse_rst(
+                    state=state,
+                    text=rst_input,
+                )
+                if isinstance(x, sphinx.addnodes.desc)
+            ]
     finally:
         env.app.disconnect(listener_id)
 
@@ -679,7 +699,7 @@ def _generate_group_summary(
             state,
             toc_entries,
             options={"hidden": True},
-            source_path="sphinx-immaterial-python-apigen",
+            source_path=__name__,
         )
     )
     return nodes
@@ -760,7 +780,12 @@ class PythonApigenEntityPageDirective(sphinx.util.docutils.SphinxDirective):
     def run(self) -> List[docutils.nodes.Node]:
         entities = _get_api_data(self.env).entities
         object_name = self.arguments[0]
-        entity = entities[object_name]
+        entity = entities.get(object_name)
+        if entity is None:
+            logger.error(
+                "Undefined Python entity: %r", object_name, location=self.get_location()
+            )
+            return []
 
         objtype = entity.objtype
         objdesc = _generate_entity_desc_node(
@@ -1298,52 +1323,6 @@ def _get_documenter_members(
             )
 
 
-_ENTITY_PAGE_INITIAL_COMMENT = (
-    "..\n  DO NOT EDIT. GENERATED by sphinx_immaterial.apidoc.python.apigen.\n"
-)
-
-
-def _is_generated_file(rst_path: str) -> bool:
-    try:
-        if os.path.islink(rst_path) or not os.path.isfile(rst_path):
-            return False
-        content = pathlib.Path(rst_path).read_text(encoding="utf-8")
-        return content.startswith(_ENTITY_PAGE_INITIAL_COMMENT)
-    except:  # pylint: disable=bare-except
-        return False
-
-
-def _write_member_documentation_page(
-    app: sphinx.application.Sphinx, entity: _ApiEntity
-) -> None:
-    """Writes the RST file that documents the given entity.
-
-    This simply writes a `python-apigen-entity-page` directive to the generated
-    file.  The actual documentation is generated by that directive.
-
-    :param app: Sphinx application object.
-    :param entity: Entity to document.
-    """
-
-    rst_path = os.path.join(app.srcdir, entity.docname + ".rst")
-
-    content = _ENTITY_PAGE_INITIAL_COMMENT
-    # Suppress "Edit this page" link since the page is generated.
-    content += "\n\n:hide-edit-link:\n\n"
-    content += sphinx_utils.format_directive(
-        "python-apigen-entity-page",
-        entity.object_name,
-    )
-    if os.path.exists(rst_path):
-        logger.error(
-            "Generated documentation page for %r would overwrite existing source file %r",
-            entity.object_name,
-            rst_path,
-        )
-        return
-    pathlib.Path(rst_path).write_text(content, encoding="utf-8")
-
-
 class SplitAutodocRstOutput(NamedTuple):
     directive: str
     options: Dict[str, str]
@@ -1505,6 +1484,8 @@ class _ApiEntityCollector:
 
         split_result = _split_autodoc_rst_output(rst_strings)
 
+        split_result.options.pop("module", None)
+
         overload_id: Optional[str] = None
         if entry.overload is not None:
             overload_id = entry.overload.overload_id
@@ -1581,17 +1562,10 @@ def _get_docname(
 ):
 
     docname = _get_base_docname(output_prefixes, documented_full_name)
-
-    if case_insensitive_filesystem:
-        name_hash = hashlib.sha256(
-            os.path.basename(docname).encode("utf-8")
-        ).hexdigest()[:8]
-        docname += f"-{name_hash}"
-
     if overload_id:
         docname += f"-{overload_id}"
 
-    return docname
+    return apigen_utils.make_unique_docname(docname, case_insensitive_filesystem)
 
 
 def _assign_documented_full_names(
@@ -1628,8 +1602,10 @@ def _assign_documented_full_names(
             # Parent is a module.
             parent_documented_name = parent_ref.parent_canonical_object_name
             entity.top_level = True
+            entity.options["module"] = parent_ref.parent_canonical_object_name
         else:
             parent_documented_name = get_documented_full_name(parent_entity)
+            entity.options["module"] = parent_entity.options["module"]
         documented_full_name = parent_documented_name + "." + parent_ref.name
         entity.documented_full_name = documented_full_name
         entity.documented_name = parent_ref.name
@@ -1655,18 +1631,6 @@ def _assign_documented_full_names(
         entities[entity.object_name] = entity
 
 
-def _is_case_insensitive_filesystem(path: str) -> bool:
-    suffix = secrets.token_hex(16)
-    temp_path = path + suffix + "a.rst"
-    try:
-        pathlib.Path(temp_path).write_text(
-            _ENTITY_PAGE_INITIAL_COMMENT, encoding="utf-8"
-        )
-        return os.path.exists(path + suffix + "A.rst")
-    finally:
-        os.remove(temp_path)
-
-
 def _builder_inited(app: sphinx.application.Sphinx) -> None:
     """Generates the rST files for API members."""
 
@@ -1702,21 +1666,14 @@ def _builder_inited(app: sphinx.application.Sphinx) -> None:
             canonical_object_name=module_name,
         )
 
-    # Prepare output directories and determine if the filesystem is case
-    # insensitive.
-    seen_output_dirs = set()
-    case_insensitive_filesystem = app.config.python_apigen_case_insensitive_filesystem
-    for output_path in apigen_modules.values():
-        output_dir = os.path.dirname(os.path.join(app.srcdir, output_path))
-        if output_dir in seen_output_dirs:
-            continue
-        seen_output_dirs.add(output_dir)
-        os.makedirs(output_dir, exist_ok=True)
-        if case_insensitive_filesystem is None:
-            if _is_case_insensitive_filesystem(os.path.join(app.srcdir, output_path)):
-                case_insensitive_filesystem = True
-    if case_insensitive_filesystem is None:
-        case_insensitive_filesystem = False
+    writer = apigen_utils.GeneratedDocumentWriter(
+        app=app,
+        case_insensitive_filesystem=app.config.python_apigen_case_insensitive_filesystem,
+        output_prefixes=list(apigen_modules.values()),
+        generator_module=__name__,
+    )
+
+    writer.prepare_output_directories()
 
     default_groups = [
         (re.compile(pattern), group)
@@ -1733,47 +1690,30 @@ def _builder_inited(app: sphinx.application.Sphinx) -> None:
         default_groups=default_groups,
         default_order=default_order,
         output_prefixes=apigen_modules,
-        case_insensitive_filesystem=case_insensitive_filesystem,
+        case_insensitive_filesystem=writer.case_insensitive_filesystem,
     )
 
-    # Remove stale generated files.
-    srcdir = app.srcdir
-    for _, output_path in app.config.python_apigen_modules.items():
-        glob_pattern = os.path.join(srcdir, output_path)
-        for p in glob.glob(os.path.join(srcdir, output_path + "*.rst"), recursive=True):
-            if not _is_generated_file(p):
-                continue
-            try:
-                os.remove(p)
-            except OSError as e:
-                logger.warning("Failed to remove stale generated file %r: %s", p, e)
-
-    # Write page for each entity.
-    all_pages: Dict[str, str] = {}
-
     alphabetical = app.config.python_apigen_order_tiebreaker == "alphabetical"
 
-    for object_name, entity in data.entities.items():
-        if object_name != entity.object_name:
-            # Alias
-            continue
+    def get_pages():
+        for object_name, entity in data.entities.items():
+            if object_name != entity.object_name:
+                # Alias
+                continue
 
-        data.sort_members(entity.members, alphabetical=alphabetical)
+            data.sort_members(entity.members, alphabetical=alphabetical)
 
-        rst_path = entity.docname + ".rst"
-        if rst_path in all_pages:
-            logger.error(
-                "Both %r and %r map to generated path %r",
-                all_pages[rst_path],
+            if entity.top_level:
+                group_id = docutils.nodes.make_id(entity.group_name)
+                data.top_level_groups.setdefault(group_id, []).append(entity.parents[0])
+
+            content = sphinx_utils.format_directive(
+                "python-apigen-entity-page",
                 entity.object_name,
-                rst_path,
             )
-            continue
-        _write_member_documentation_page(app, entity)
-        all_pages[rst_path] = entity.object_name
-        if entity.top_level:
-            group_id = docutils.nodes.make_id(entity.group_name)
-            data.top_level_groups.setdefault(group_id, []).append(entity.parents[0])
+            yield entity.docname, entity.object_name, content
+
+    writer.write_files(get_pages())
 
     for members in data.top_level_groups.values():
         data.sort_members(members, alphabetical=alphabetical)
@@ -1877,4 +1817,10 @@ def setup(app: sphinx.application.Sphinx):
         types=(bool,),
         rebuild="env",
     )
+    app.add_config_value(
+        "python_apigen_rst_prolog", types=(str,), default="", rebuild="env"
+    )
+    app.add_config_value(
+        "python_apigen_rst_epilog", types=(str,), default="", rebuild="env"
+    )
     return {"parallel_read_safe": True, "parallel_write_safe": True}
