@@ -153,7 +153,7 @@ import sphinx.environment.adapters.toctree
 import sphinx.util.docutils
 import sphinx.util.osutil
 
-from . import apidoc_formatting
+from .apidoc import object_description_options
 
 StandaloneHTMLBuilder = sphinx.builders.html.StandaloneHTMLBuilder
 
@@ -434,10 +434,10 @@ def _add_domain_info_to_toc(
             continue
         domain = env.domains[objinfo.domain_name]
         label = domain.get_type_name(domain.object_types[objinfo.objtype])
-        options = apidoc_formatting.get_object_description_options(
+        options = object_description_options.get_object_description_options(
             env, objinfo.domain_name, objinfo.objtype
         )
-        tooltip = apidoc_formatting.format_object_description_tooltip(
+        tooltip = object_description_options.format_object_description_tooltip(
             env, options, objinfo.name, objinfo.synopsis
         )
         toc_icon_text = options["toc_icon_text"]
@@ -768,6 +768,7 @@ def _html_page_context(
             "mdx_configs": {
                 "toc": {"title": toc_title},
             },
+            "extra": {"consent": theme_options.get("consent")},
         }
     )
 
@@ -796,6 +797,10 @@ def _html_page_context(
         page["meta"]["hide"].append("toc")
     if "hide-navigation" in meta:
         page["meta"]["hide"].append("navigation")
+    if "hide-footer" in meta:
+        page["meta"]["hide"].append("footer")
+    if "hide-feedback" in meta:
+        page["meta"]["hide"].append("feedback")
     if context.get("next"):
         page["next_page"] = {
             "title": markupsafe.Markup.escape(
